#ifndef _h_DOSINFO_
#define _h_DOSINFO_

/* This header based on Ralf Brown's Interrupt List 53 */

#ifndef _h_TYPES_
# include <cv/types.h>
#endif

#pragma option -a-

struct	MCB;		/* size == 16 */
struct	PSP;		/* size == 256 */
union	DevAttr;	/* size == 2 */
struct	DevHdr;		/* size == 18 */
struct	DevCDROM;	/* size == sizeof(DevHdr) + 10 */
struct	DevInterlnk;	/* size == sizeof(DevHdr) + 335 */
struct	LZSinfo;	/* size == 60 */
struct	DevStacker;	/* size == sizeof(DevHdr) + 120 */
struct	DevStacker4;	/* size == sizeof(DevStacker) + 220 + sizeof(LZSinfo) */

struct	BPB;		/* size == 13 */
struct	Dos2BPB;	/* size == sizeof(BPB) + 6 */
struct	Dos3BPB;	/* size == sizeof(BPB) + 8 [+ 11 dummy] */
struct	Dos4BPB;	/* size == sizeof(Dos3BPB) + 4 */
struct	Dos4BPBe;	/* size == sizeof(Dos4BPB) + 11 */
struct	ExtBPB;		/* size == sizeof(Dos4BPB) + 28 */

struct	DPB;		/* size == 33 */
struct	CDS;		/* size == 88 */

struct	Dos2SFT;	/* size = 46 */
struct	Dos3SFT;	/* size = 62 */
struct	Dos3xSFT;	/* size = 59 */
struct	Dos4SFT;	/* size = 65 */

struct	DospreLoL;	/* size == 2 */
struct	Dos3preLoL;	/* size == sizeof(DospreLoL) + 6 */
struct	Dos3xpreLoL;	/* size == sizeof(Dos3preLoL) + 16 */

struct	DosLoL;		/* size == 16 */
struct	Dos2LoL;	/* sizeof(DosLoL) + 7 + sizeof(DevHdr) */
struct	Dos3LoL;	/* sizeof(DosLoL) + 24 + sizeof(DevHdr) */
struct	Dos3xLoL;	/* sizeof(DosLoL) + 19 + sizeof(DevHdr) */
struct	Dos4LoL;	/* sizeof(Dos3xLoL) + 18 */
struct	DosBoxLoL;	/* sizeof(DosLoL) + 18 + sizeof(DevHdr) */


/*---------------------------------------------------------*/
/* internal structures */
struct DOS{
	struct LRUFCB{
	    word	cache,	/* LRU counter for FCB caching/last IO */
			opens;	/* LRU counter for FCB opens/last open */
	};
};


/*=========================================================*/
/* MCB: Memory Control Block */
/* This structure is used internally by DOS when it allocates memory. */
/* An undocumented way to find the first MCB to use DosLoL */

struct MCB{
    byte	signature;	/* 'M'=valid, 'Z'=last block in list */
    word	ownerID;	/* seg @ PSP of owner (0=owns itself) or */
			/* special flag: 6=DR-DOS XMS UMB, 7=DR-DOS excluded */
			/* upper memory, 8=belongs to DOS, 0xfffx=386MAX) */
    word	sz;		/* allocation size in paragraphs (16 bytes) */
    byte	dummy[3];
    char	name[8];	/* ASCIIZ name of owner (DOS 4.0+) */
}; /* MCB */


/*---------------------------------------------------------*/
/* PSP: Program Segment Prefix */
/* This structure is created for each program that DOS executes. Can be */
/* obtained via DOS Fn 0x51 */

struct PSP{
    word	int20;		/* Int20h instruction (0xCD 0x20) */
				/*   (old way to exit) */
    word	next;		/* seg @ just beyond end of pgm */
    byte	dummy1;
    byte	dispatch[5];	/* FAR CALL to DOS Fn dispatcher (obsolete) */
    void far	*terminate;	/* (on exit, DOS copies this to Int22h vec) */
    void far	*ctrlBreak;	/* Ctrl-Break handler address (see Int23h) */
    void far	*critErr;	/* Critical Error handler addr (see Int24h) */
    word	parent;		/* seg @ parent's PSP (undocumented) */
    byte	dummy2[20];
    word	envseg;		/* seg @ DOS environment */
    byte	dummy3[46];	/* (handle table, et al.) */
    byte	FCB1[16];	/* an unopened FCB for 1st cmd parameter */
    byte	FCB2[20];	/* an unopened FCB for 2nd cmd parameter */
    byte	cmdlineLen;
    char	cmdline[127];
}; /* PSP */


/*---------------------------------------------------------*/
/* Device Attributes */
union DevAttr{
	struct{ unsigned
		isStdin		:1,
		isStdout	:1,
		isStdnul	:1,
		isClock		:1,
		isFastOut	:1,
				:1,
		genericIOCTL	:1,	/* supports Generic IOCTL */
		queryIOCTL	:1,
				:3,
		devOpenClose	:1,
				:1,
		outTillBusy	:1,
		IOCTLRW		:1,	/* supports IOCTL Read & IOCTL Write */
		isChar		:1;	/* 1 */
	} charDev;
	struct{ unsigned	:1,
		bigDisk		:1,	/* supports >32M partitions */
				:4,
		genericIOCTL	:1,	/* supports Generic IOCTL & */
					/*   Get/Set Logical Device */
		queryIOCTL	:1,
				:3,
		devOpenClose	:1,	/* supports device open/close & */
					/*   Removable Media */
		remote		:1,	/* device is remote */
		nonFAT		:1,	/* not need FAT with BPB (non-IBM) */
		IOCTLRW		:1,	/* supports IOCTL Read & IOCTL Write */
		isChar		:1;	/* 0 */
	} blockDev;
}; /* DevAttr */


/* Device Header Layout */
/* The file with device driver begins with this structure and this data */
/* remains in memory after installation of the driver. The address of the */
/* start of the device driver chain may be found via DosLoL */

struct DevHdr{
    DevHdr far	*next;		/* offset=0xffff for last driver */
    DevAttr	attr;
    word	offStrategy,	/* offset @ device Strategy routine */
		offInterrupt;	/* offset @ device Interrupt routine */
    union{ struct{
		byte	units;		/* # supported subunits (drives) */
		char	sign[7];	/* normally unused */
	} block;		/* [block device] */
	char	name[8];	/* [character device] blank-padded name */
    };
}; /* DevHdr */


/* Extra fileds for CD-ROM */
struct DevCDROM :DevHdr{
    word	dummy;
    char	drvLetter;	/* CD-ROM drive letter */
    byte	units;		/* # CD-ROM units */
    char	signature[6];	/* "MSCDnn" */
}; /* DevCDROM */

struct DevInterlnk :DevHdr{		/* name="NUL2" */
    byte	dummy1[165];
    char	filename[67];	/* fully qualified Interlnk filename */
    byte	dummy2[6];
    void far	*pName;		/* @ of [filename] above */
    byte	dummy3[8];
    byte	firstRemote,	/* first redirected drive # */
		firstLocal,	/* first local drive # (0=A:) */
		printer;	/* 1=printer redirected */
    byte	dummy4;
    byte	lptStatus[3],	/* LPT1-3 status (0xff=invalid) */
		remote[26],	/* remote drive number (0=A:, 0xfe=unused) */
			/* (remote drive remote[i] maps to [local]+i+'A') */
		dummy5[26],	/* always 0xfe */
		dummy6[26];	/* always 0xff */
};

struct LZSinfo{
    char	signature[6];	/* "LZSAPI" */
    word	ver;		/* 200 for Stacker4/Stacker Anywhere */
    char	vendor[4];	/* vendor signature ("STAC" = Stac */
				/*   Electronics, Inc.) */
    byte	dummy1[6];
    word	status;		/* (bit 1=bussy/error, bit 11=max compressor */
				/*   present) */
    void far	*comp3;		/* @ Stacker 3.x compression procedure */
    void far	*decomp3;	/* @ Stacker 3.x decompression proc */
    byte	dummy2[4];
    void far	*comp3inc;	/* @ incremental Stacker 3.x compression proc */
    byte	dummy3[4];
    void far	*decomp3inc;	/* @ incremental Stacker 3.x decompression */
    byte	dummy4[4];
    void far	*comp4;		/* @ Stacker 4 compression procedure */
    byte	dummy5[4];
    void far	*decomp4;	/* @ Stacker 4 decompression procedure */
};

struct DevStacker :DevHdr{		/* block.sign="STAC-CD" */
    byte	dummy1[7];
    byte	one;		/* always 1 */
    word	signature;	/* 0xa55a */
    word	ver;		/* Stacker version * 100 */
    word	offVolInfoTbl;	/* @ volume-specific information offset table */
    byte	dummy2[56];
    byte	volNo;		/* volume # (after Int21/AX=4404, AX=4408) */
				/*   use to index into VolInfoTbl */
    byte	dummy3[19];
    char	swap[4];	/* "SWAP" */
    byte	mapping[26];	/* drive mapping table (only for drives */
				/*   swapped by SSWAP.COM) */
};

struct DevStacker4 :DevStacker{
    byte	dummy1[40];
    char	swap2[4];	/* "SWP2" */
    byte	mapping2[26];
    byte	dummy2[150];
    LZSinfo	LZS;
};


/*---------------------------------------------------------*/
/* BPB: BIOS Parameter Block */
/* This is subset of the data, found in the BootSector used by device */
/* drivers. The BuildBPB device driver cmd expects the driver to fill a */
/* buffer in this layout */

struct BPB{
    word	sectorSz;	/* bytes per sector (usually 512) */
    byte	sectsPerCluster;/* sectors per cluster */
    word	reservedSects;	/* boot and "reserved" sectors (usually 1) */
    byte	FATcnt;		/* # FATs (usually 2) */
    word	rootEntries,	/* max # of 32-byte entries allowed in root */
		totalSects;	/* # sectors in media (0 means >32M, */
				/*   so use bigTotalSects) */
    byte	mediaId;	/* media descriptor */
    word	sectPerFAT;	/* sectors per FAT */
}; /* BPB */

struct Dos2BPB :BPB{	/* DOS 2.13 */
    word	sectsPerTrack, heads,
		hiddenSects;
}; /* Dos2BPB */

struct Dos3BPB :BPB{	/* DOS 3.0+ (not count rest "byte dymmy[11];") */
    word	sectsPerTrack, heads;
    ulong	hiddenSects;
}; /* Dos3BPB */

struct Dos4BPB :Dos3BPB{		/* DOS 4.0+ */
    ulong	bigTotalSects;	/* sectors count for partitions >32M if */
				/*   totalSects=0 */
}; /* Dos4BPB */

struct	Dos4BPBe :Dos4BPB{		/* not for European MS-DOS 4.0 */
    byte	dummy[6];
/*---------- not for DR-DOS 5+ */
    word	cyls;		/* # cylinders */
    byte	devtype;
    word	devattr;	/* (removable or not, etc) */
}; /* Dos4BPBe */

struct ExtBPB :Dos4BPB{
    ulong	bigFATsects;	/* sectors per FAT if FATsects=0 */
    word	extflags;	/* (bits 3-0=the 0-based FAT number of the */
			/* active FAT, bit 7=do not mirror active FAT to */
			/* inactive FATs) */
    word	ver;		/* FS version (high byte=major, low=minor) */
    ulong	cluster,	/* starting cluster # of root dir */
		FSinfoSect;	/* FS information sector # */
    word	dummy[6];
}; /* ExtBPB */


/*---------------------------------------------------------*/
/* DPB: Drive Parameter Block */
/* The DPB describes a drive and the medium in the drive. Can be obtained */
/* via DOS Fn 0x32 & 0x1f. DOS 2.x at offset 0x1C contain some CDS fields */

struct DPB{
    byte	driveNo,	/* drive # (0=A, 1=B ...) */
		unitNo;		/* sub-unit # */
    word	sectorSz;	/* bytes per sector */
    byte	clustMask,	/* sectors per cluster-1 (max sector in cluster) */
		clustShift;	/* cluster-to-sector shift (cluster is 2^sectors) */
    word	FATstart;	/* sector # of start of first FAT (also */
				/*   # "reserved" sectors, usually 1) */
    byte	FATcnt;		/* # FATs (usually 2) */
    word	rootEntries,	/* # of entries allowed in root directory */
		dataStart,	/* sector # of cluster #2 (first data cluster); */
			/* cluster #n starts at sector #=firstData */
			/* + ((n - 2) * (clustMask + 1)) */
		lastCluster;	/* total clusters+1 (# of highest cluster) */
  	union{ struct{
    byte	sectsPerFAT;	/* sectors per FAT */
    word	rootStart;	/* starting sector # of root dir */
    DevHdr far	*pDevice;	/* @ header for this device driver */
    byte	mediaId,	/* media descriptor (see BPB) */
		accessed;	/* 0xff=medium in drive has not been accessed */
    DPB far	*next;		/* offset=0xffff for last */
    word	lastUsedCluster,/* last allocated cluster */
		freeClusts;	/* MAY contain # free clusters (0xffff mean */
				/*   this field has not been calculated yet) */
	} Dos2;		/* DOS 2.x-3.x */
	struct{
    word	sectsPerFAT;	/* sectors per FAT */
    word	rootStart;	/* starting sector # of root dir */
    DevHdr far	*pDevice;	/* @ header for this device driver */
    byte	media,		/* media descriptor (see BPB) */
		accessed;	/* 0xff=medium in drive has not been accessed */
    DPB far	*next;		/* offset=0xffff for last */
    word	lastUsedCluster,/* last allocated cluster */
		freeClusts;	/* MAY contain # free clusters (0xffff mean */
				/*   this field has not been clculated yet) */
	} Dos4;		/* DOS 4.0-6.0 */
    };
}; /* DPB */


/*---------------------------------------------------------*/
/* CDS: Current Directory Structure */
/* This is the undocumented structure DOS maintains for each logical drive */
/* (as set via the lastdrive= in config.sys). Use DosLoL to */
/* locate the first of these structures and to find how many there are */

struct CDS{
    char	path[67];	/* ASCIIZ "d:\path" of current dir; on SUBST */
			/* or JOIN drives, this is the true name of the dir */
			/* on the actual drive in which it resides */
    word	flag;		/* drive attributes (bit 15=uses network */
			/* redirector, bit 14=physical drive [00=invalid, */
			/* 11=IFS], bit 13=JOIN'ed, bit 12=SUBST'ed, 7=hidden */
			/* remote drive or CD-ROM by MSCDEX) */
    DPB far	*thisDPB;	/* @ DPB for this drive */
    union{ struct{
	void far *IFS;		/* @ redirector or REDIRIFS rec, or ffff:ffff */
	word	netUserCode;	/* network user code from Int21h/AX=5F03 */
	} net;
	word	cluster;	/* starting cluster # of current dir */
				/*   (0=root, 0xffff=never accessed) */
    };
    word	offSlash;	/* offset of first '\' in path (to skip d:) */
/*---------- DOS 4.0+ */
    byte	devtype;	/* 4=network drive */
/**/void far	*IFSdrv;	/* @ IFS driver (DOS 4) or redirector block */
				/*   (DOS 5+) for this drive, 0=native DOS */
    word	dummy;
}; /* CDS */


/*---------------------------------------------------------*/
/* SFT: System File Table */
struct Dos2SFT{		/* DOS 2.x */
    Dos2SFT far	*next;		/* offset=0xffff for last */
    word	files;		/* # files in this table */
    struct{
	byte	handles,	/* # file handles referring to this file */
		openmode,
		attr,
		drive;		/* 0=character device, 1=A, ... */
	char	name[11];	/* FCB format (no path/period, blank-padded) */
	word	dummy1[2];
	ulong	fsize;		/* ??? */
	word	fdate,		/* date and time both in packed format */
		ftime;
	byte	devattr;
	union{	DevHdr far *pDevice;	/* character device */
		struct{ word	start,		/* starting cluster of file */
				last;		/* relative cluster in file */
						/* of last cluster accessed */
			} BLK;		/* block device */
	};
	word	absolute;	/* absolute cluster # of current cluster */
	word	dummy2;
	ulong	fpos;		/* current file position */
    } item[1];
}; /* Dos2SFT */

struct Dos3SFT{		/* DOS 3.0 */
    Dos3SFT far	*next;		/* offset=0xffff for last */
    word	files;		/* # files in this table */
    struct{
	word	handles,	/* # file handles referring to this file */
		openmode;	/* (bit 15=this file open via FCB) */
	byte	attr;
	word    devattr;	/* (bit 15=remote file, bit 14=don't set file */
			/* date/time on close, 12=don't inherit on EXEC, */
			/* bits 5-0=drive # for disk files) */
	union{	DevHdr far *pDevice;	/* [character device] */
		DPB far *pDPB;		/* [block device] */
	};
	word	start;		/* starting cluster */
	word	ftime,		/* date and time both in packed format, */
		fdate;		/*   not used for character devices in DR-DOS */
	ulong	fsize;
	union{	ulong	fpos;	/* [SFT] current offset in file */
		DOS::LRUFCB LRU;/* [FCBtbl] */
	};
	word	last,		/* relative cluster in file of last */
				/*    cluster accessed */
		absolute;	/* absolute cluster # of current cluster */
	word	direntrySect,	/* # of sector containing direntry */
		offDirentry;	/* byte offset of direnty within sector */
	char	name[11];	/* FCB format (no path/period, blank-padded) */
	Dos3SFT far *prev;	/* [share.exe] previous SFT sharing same file */
	word	machine,	/* [share.exe] network machine/VDM # which */
				/*    open file */
		ownerPSP,
		share;		/* offset in SHARE.EXE code seg of share */
				/*   record */
	word	dummy;
    } item[1];
}; /* Dos3SFT */

struct Dos3xSFT{	/* DOS 3.1-3.3x, DR-DOS 5.0-6.0 */
    Dos3xSFT far *next;		/* offset=0xffff for last */
    word	files;		/* # files in this table */
    struct{
	word	handles,	/* # file handles referring to this file */
		openmode;	/* (bit 15=this file open via FCB) */
	byte	attr;
	word    devattr;	/* Also bit 15=remote file, bit 14=don't set */
			/* file date/time on close, 12=don't inherit on EXEC */
	union{	DevHdr far *pDevice;	/* [character device] */
		DPB far *pDPB;		/* [block device] */
	};
	word	start;		/* [local] starting cluster */
	word	ftime,		/* date and time both in packed format, */
		fdate;		/*   not used for character devices in DR-DOS */
	ulong	fsize;
	union{	ulong	fpos;	/* [SFT] current offset in file */
		DOS::LRUFCB LRU;/* [FCBtbl] */
	};
	word	last,		/* relative cluster in file of last */
				/*    cluster accessed */
		absolute;	/* absolute cluster # of current cluster */
	word	direntrySect;	/* # of sector containing direntry */
	byte	direntry;	/* # direntry within sector (byte offset/32) */
	char	name[11];	/* FCB format (no path/period, blank-padded) */
	Dos3xSFT far *prev;	/* [share.exe] previous SFT sharing same file */
	word	machine,	/* [share.exe] network machine/VDM # which */
				/*    open file */
		ownerPSP,
		share;		/* offset in SHARE.EXE code seg of share */
				/*   record (0=none) */
    } item[1];
}; /* Dos3xSFT */

struct Dos4SFT{		/* DOS 4.0+ */
    Dos4SFT far	*next;		/* offset=0xffff for last */
    word	files;		/* # files in this table */
    struct{
	word	handles,	/* # file handles referring to this file */
		openmode;	/* (bit 15=this file open via FCB) */
	byte	attr;
	word    devattr;	/* Also bit 15=remote file, bit 14=don't set */
			/* file date/time on close, 13=named pipe, 12=don't */
			/* inherit on EXEC, 11=network spooler, 7=device/file */
	union{	DevHdr far *pDevice;	/* [character device] */
		DPB far *pDPB;		/* [block device] */
	};
	word	start;		/* [local] starting cluster */
	word	ftime,		/* date and time both in packed format, */
		fdate;		/*   not used for character devices in DR-DOS */
	ulong	fsize;
	union{	ulong	fpos;	/* [SFT] current offset in file */
		DOS::LRUFCB LRU;/* [FCBtbl] */
	};
	union{ struct{
		word	last;		/* relative cluster in file of last */
					/*    cluster accessed */
		ulong	direntrySect;	/* # of sector containing direntry */
		byte	direntry;	/* # direntry within sector */
					/*   (byte offset/32) */
		} local;
		void far *IFS;		/* [network] REDIRIFS record */
	};
	char	name[11];	/* FCB format (no path/period, blank-padded) */
	Dos4SFT far *prev;	/* [share.exe] previous SFT sharing same file */
	word	machine,	/*  [share.exe] network machine/VDM # which */
				/*    open file */
		ownerPSP,
		share;		/* offset in SHARE.EXE code seg of share */
				/*   record (0=none) */
	word	absolute;	/* absolute cluster # of current cluster */
	void far *IFSdrv;	/* 0=native DOS */
    } item[1];
}; /* Dos4SFT */


/*---------------------------------------------------------*/
/* DOS List of Lists */
/* Can be obtained by the undocumented DOS Fn 0x52 */

struct DospreLoL{
    word	baseMemSeg;	/* seg of first MCB */
}; /* DospreLoL */

struct Dos3preLoL{	/* DOS 3.0+ */
    void far	*currbuf;	/* current disk buffer */
    word	offInput;	/* @ in DOS data seg of unread CON handle IO */
				/*   line-buffered input (0=no unread) */
    word	baseMemSeg;	/* seg of first MCB */
}; /* Dos3preLoL */

struct Dos3xpreLoL{	/* DOS 3.1+ */
    word	cx;		/* contents of CX from Int21h/AX=5E01 */
    DOS::LRUFCB	LRU;
    void far	*OEMhandler;	/* @ OEM fn handler (ffff:ffff=not available) */
    word	offDOSret;	/* @ in DOS CS of code to return from Int21h */
    word	shareCount,	/* sharing retry count */
		shareDelay;	/* sharing retry delay */
    void far	*currbuf;	/* current disk buffer */
    word	offInput;	/* @ in DOS data seg of unread CON handle IO */
				/*   line-buffered input (0=no unread) */
    word	baseMemSeg;	/* seg of first MCB */
}; /* Dos3xpreLoL */

struct DosLoL{
    DPB far	*baseDPB;	/* @ first Drive Parameter Block */
    void far	*firstSFT;	/* @ first System File Table */
    DevHdr far	*pClockDev;	/* @ active CLOCK$ device header */
    DevHdr far	*pConDev;	/* @ active CON device header */
}; /* DosLoL */

struct Dos2LoL :DosLoL{			/* DOS 2.x */
    byte	drives;		/* # logical drives */
    word	maxSectSz;	/* max block size for block devices */
/**/void far	*pCache;	/* @ DOS's internal disk buffers */
    DevHdr	NulDev;		/* NUL device (start of device driver chain) */
}; /* Dos2LoL */

struct Dos3LoL :DosLoL{			/* DOS 3.0 */
    byte	drives;		/* # block devices */
    word	maxSectSz;	/* max block size for block devices */
/**/void far	*pCache;	/* @ DOS's internal disk buffers */
    CDS far	*CDStbl;	/* @ CDS[lastdrive] structure array */
    byte	lastdrive;	/* config.sys lastdrive= value */
    void far	*workspace;	/* @ STRING= workspace area */
    word	workspaceSz;	/* config.sys STRING= value */
    Dos3SFT far	*FCBtbl;	/* @ system FCB table */
    word	protectedFCB;	/* # protected FCBs (config.sys fcbs= second */
				/*   argument) */
    DevHdr	NulDev;		/* NUL device (start of device driver chain) */
}; /* Dos3LoL */

struct Dos3xLoL :DosLoL{		/* DOS 3.1-3.3 */
    word	maxSectSz;	/* maximum sector size for block devices */
/**/void far	*pCache;	/* @ DOS's internal disk buffers */
    CDS far	*CDStbl;	/* @ CDS[lastdrive] structure array */
    void far	*FCBtbl;	/* @ system FCB table (see DosxxSFT) */
    word	protectedFCB;	/* # protected FCBs (config.sys fcbs= second */
				/*   argument) */
    byte	drives,		/* # drives in system */
		lastdrive;	/* # available drive letters (largest of 5, */
			/* installed block devices and config.sys lastdrive= */
			/* value). Also size of CDS array */
    DevHdr	NulDev;		/* NUL device (start of device driver chain) */
    byte	join;		/* # JOIN'ed drives */
}; /* Dos3xLoL */

struct Dos4LoL :Dos3xLoL{		/* DOS 4.0+ */
    word	listOfSpecName;	/* offset in IBMDOS CS of list of special */
				/*   program names (always 0 for DOS 5) */
    union{ struct{
		void far *IFSutility,	/* default action for resident IFS */
			 *IFSchain;
	} Dos4;
	struct{
		void far *setver;	/* @ SETVER program list or 0 */
		word	offA20fix;	/* [DOS=HIGH] offset in DOS CS of */
					/* function to fix A20 control */
		word	lastPSP;	/* PSP of last EXECed program if */
					/* DOS in HMA, 0 if low */
	} Dos5;
    };
    word	buffers,	/* config.sys buffers= value (rounded up to */
				/*   30x if in EMS) */
		ahead;		/* config.sys buffers= second value */
				/*   (lookahead buffers) */
    byte	bootDrv;	/* 1=A:, ... */
    byte	flag386;	/* 1=use DWORD moves (80386+) */
    word	extmemSz;	/* extended memory size in KB */
}; /* Dos4LoL */

struct DosBoxLoL :DosLoL{		/* Windows NT DOS Box */
    byte	dummy1[6];
    CDS far	*CDStbl;	/* @ CDS[lastdrive] structure array */
    byte	dummy2[6];
    byte	drives,		/* # drives in system */
		lastdrive;	/* # local drive letters (= # drives). */
				/*   Also size of CDS array */
    DevHdr	NulDev;		/* NUL device (start of device driver chain) */
}; /* DosBoxLoL */


#pragma option -a.

#endif /* _h_DOSINFO_ */
